@@ -1,10 +1,12 @@
-static char *rcsid = "$Header: /plaid/homes/zeus/Blind-2.46/RCS/Blind.c,v 2.46 1997/04/18 12:06:12 zeus Exp zeus $";
+static char *rcsid = "$Header: /plaid/homes/zeus/Blind-2.47/RCS/Blind.c,v 2.47 1998/04/15 21:34:17 zeus Exp zeus $";
 
 /*
  * $Log: Blind.c,v $
- * Revision 2.46  1997/04/18 12:06:12  zeus
+ * Revision 2.47  1998/04/15 21:34:17  zeus
  * *** empty log message ***
  *
+ * Revision 2.46.0.1  1997/05/27 21:03:51  zeus
+ * Revision 2.46  1997/04/18 12:06:12  zeus
  * Revision 2.45  1996/12/31  23:40:54  zeus
  * Revision 2.44  1995/12/31  19:45:18  zeus
  * Revision 2.43  1995/03/20  01:43:00  zeus
@@ -43,6 +45,7 @@ static char *rcsid = "$Header: /plaid/homes/zeus/Blind-2.46/RCS/Blind.c,v 2.46 1
 #include "new-game.h"
 #include "turn-g.h"
 #include "exit.h"
+#include "report-g.h"
 
 int main (int argc, char *argv[]);
 void blind_rcsid (void);
@@ -95,7 +98,7 @@ main (int argc, char *argv[])
 #endif
   int i;
   player *P;
-  char fileext[MAXTEXT];
+  char fileext[MAXTEXT * 2];
 
 #endif
 
@@ -119,7 +122,7 @@ main (int argc, char *argv[])
     /* Use supplied comd line args as commands, if provided. */
     if (argc > 1)
     {
-      strcpy (buf, argv[1]);
+      hstrncpy (buf, argv[1], MAXSTRLEN);       /* Bugfix 5/12/97 */
       argc--;
       argv++;
     }
@@ -175,7 +178,7 @@ main (int argc, char *argv[])
       getstr ();
       if (buf[0])
       {
-        strcpy (fileext, buf);
+        hstrncpy (fileext, buf, MAXTEXT);
         if (buf[0] != '.')
           sprintf (fileext, ".%s", buf);
       }
@@ -228,6 +231,18 @@ main (int argc, char *argv[])
       if (pr && (pr != stdout))
         fclose (pr);
       break;
+    case 'd':                   /* Overall Battle Summary */
+      inputpr ();
+      gm_battle ();
+      if (pr && (pr != stdout))
+        fclose (pr);
+      break;
+    case 'c':                   /* Overall Bombing Summary */
+      inputpr ();
+      gm_bombing ();
+      if (pr && (pr != stdout))
+        fclose (pr);
+      break;
     case 'f':                   /* full listing for all planets */
       inputpr ();
       showfull ();
@@ -316,6 +331,8 @@ main (int argc, char *argv[])
                "* = List renamed systems\n"
                "+ = New random seed\n"
                "B = Current order status\n"
+               "C = Current Bombing Summary\n"
+               "D = Current Battle Summary\n"
                "F = Full report on all systems\n"
                "K = Kurrent players status\n"
                "L = pLanet info\n"
