@@ -1,8 +1,11 @@
-static char *rcsid = "$Header: /plaid/homes/zeus/Blind-2.46/cmds/RCS/z.c,v 2.46 1997/04/18 12:13:56 zeus Exp zeus $";
+static char *rcsid = "$Header: /plaid/homes/zeus/Blind-2.47/cmds/RCS/z.c,v 2.47 1998/04/15 21:34:50 zeus Exp zeus $";
 
 /*
  *
  * $Log: z.c,v $
+ * Revision 2.47  1998/04/15 21:34:50  zeus
+ * *** empty log message ***
+ *
  * Revision 2.46  1997/04/18 12:13:56  zeus
  * *** empty log message ***
  *
@@ -33,12 +36,12 @@ z_cmd_rcsid (void)
 void
 z_cmd (char *buf, player * P)
 {
-  char t[MAXTEXT];
-  char newentry[2*MAXTEXT+1];
+  char t[MAXSTRLEN];
+  char newentry[MAXTEXT+MAXSTRLEN+2]; /* racename:maxstrlen +1 (:) + 1 */
   int l;
   
   cmd_init(buf);
-  hstrncpy (t, next_token (), MAXTEXT);
+  hstrncpy (t, next_token (), MAXSTRLEN);
   if (t[0])
   {
     if (strchr(t,(int) ':'))
@@ -51,6 +54,7 @@ z_cmd (char *buf, player * P)
     l = strlen(newentry);
     newentry[l]=':';
     newentry[l+1]='\0';
+    /* This is safe, since hstrncpy null terminates t */
     strncat (newentry,t,MAXSTRLEN-MAXTEXT-2);
 #ifndef PREPROC /* Do NOT reset if using preprocessor! */
     if (reset_email (P->name,newentry))
@@ -67,4 +71,11 @@ z_cmd (char *buf, player * P)
   }
   extra_token();
 }
+
+#else
+void
+z_cmd (char *buf, player * P)
+{
+  error ("This command is not enabled!\n");
+}
 #endif
