@@ -1,10 +1,16 @@
-static char *rcsid = "$Header: /silver/homes/zeus/Blind-2.43/RCS/enterturn.c,v 2.43 1995/03/20 01:43:09 zeus Exp zeus $";
+static char *rcsid = "$Header: /plaid/homes/zeus/Blind-2.47/RCS/enterturn.c,v 2.47 1998/04/15 21:29:39 zeus Exp zeus $";
 
 /*
  * $Log: enterturn.c,v $
- * Revision 2.43  1995/03/20  01:43:09  zeus
+ * Revision 2.47  1998/04/15 21:29:39  zeus
  * *** empty log message ***
  *
+ * Revision 2.46  1997/04/18 12:06:12  zeus
+ * *** empty log message ***
+ *
+ * Revision 2.45  1996/12/31  23:40:54  zeus
+ * Revision 2.44  1995/12/31  19:45:18  zeus
+ * Revision 2.43  1995/03/20  01:43:09  zeus
  * Revision 2.42  1994/05/17  13:43:15  bampton
  * Revision 2.41  1994/04/08  03:11:28  bampton
  * Revision 2.40  1994/03/09  20:22:55  bampton
@@ -22,13 +28,19 @@ static char *rcsid = "$Header: /silver/homes/zeus/Blind-2.43/RCS/enterturn.c,v 2
 
 #define NOTIMEB
 #define IO
-#include "protos/common.h"
-#include "protos/prototypes.h"
-#include "protos/enterturn.h"
-#include "protos/enterturn-g.h"
-#include "protos/cmds-g.h"
-#include "protos/misc-g.h"
-#include "protos/exit.h"
+#include "common.h"
+#include "prototypes.h"
+#include "enterturn.h"
+#include "enterturn-g.h"
+#include "cmds-g.h"
+#include "cmds.h"
+#include "misc-g.h"
+#include "exit.h"
+#include "options.h"
+
+#ifdef PASSWORDS
+#include <errno.h>
+#endif
 
 void enterturn_rcsid (void);
 
@@ -41,23 +53,28 @@ void enterturn_rcsid (void);
 extern player *players;
 extern shiptype *shiptypes;
 extern battle *battles;
-extern char buf[256];
+extern char buf[MAXSTRLEN];
 extern double galaxysize;
-int groupno;
 extern planet *planets;
 extern int turn;
 
 #ifdef PREPROC
-int bad = 0;
-int n_order = 0;
+int bad;
+int n_order;
+
+#include "turn-g.h"             /* For go() */
+#endif
+
+#ifdef GAMENAME
+extern char gamename[MAXTEXT];
 
-#include "protos/turn-g.h"	/* For go() */
 #endif
 
 extern planetdefense *defender;
 extern char err_tmp[];
 
-char orders[256];
+int groupno;
+char orders[MAXSTRLEN];
 
 
 void
@@ -72,15 +89,9 @@ error (char msg[])
 
 #ifdef PREPROC
   bad++;
-
-#ifdef SMART
   fprintf (stderr, "Mistake (%0d):'%s'\n", n_order, orders);
   fprintf (stderr, "%s", msg);
   return;
-#endif
-
-  printf ("          ");
-  printf ("%s", msg);
 #else
   printf ("%s\n", msg);
 #endif
@@ -92,16 +103,18 @@ error (char msg[])
 void
 append_cmd (char *order, char *cmd)
 {
-  int i, j, s;
+  int s;
 
 
-  s=strlen (order);
-  assert (s + strlen (cmd) < 255);
+  s = strlen (order);
+  assert (s + strlen (cmd) < MAXSTRLEN - 1);
 
-  order[1 + s] = '\0';
-  if (s!=0)
-  order[s] = ' ';
-  for (i = strlen (order), j = 0; (order[i] = cmd[j]); i++, j++);
+  if (s != 0)
+  {
+    order[1 + s] = '\0';
+    order[s] = ' ';
+  }
+  (void) strcat (order, cmd);
 
 }
 
@@ -146,50 +159,188 @@ cktech (group * g)
 }
 
 #ifdef PASSWORDS
+int
+reset_password (char *name, const char newentry[])
+{
+  FILE *pwf, *pwf2;
+  char pwl[2 * MAXTEXT + 1];
+  int j;
+  int matched = 0;
+
+  memset (pwl, 0, 2 * MAXTEXT + 1);
+  pwf = fopen (PASSWORDFILE, "r");
+  pwf2 = fopen ("passwords:races", "w");
+  if (pwf == NULL)
+  {
+    error ("Password file missing!\n");
+    exit (MISSINGPW);           /* Urk. No file? */
+  }
+  if (pwf2 == NULL)
+  {
+    error ("Unable to create temp password file!\n");
+    exit (MISSINGPW);           /* This is wrong, but probably will never happen? */
+  }
+  if (name[0])
+    j = strlen (name);
+  else
+    j = 0;                      /* Needed later */
+  for (;;)
+  {
+    if (feof (pwf))             /* Player name not matched in file */
+    {
+      if ((!matched) && newentry)
+        fprintf (pwf2, "%s\n", newentry);
+      fclose (pwf);
+      fclose (pwf2);
+      if (rename ("passwords:races", PASSWORDFILE))
+      {
+        sprintf (err_tmp, "Rename failed, error number %d\n", errno);
+        error (err_tmp);
+        exit (MISSINGPW);
+      }
+      return (0);
+    }
+    /* Do the EOF check first, to deal with empty file and match on last */
+    fscanf (pwf, "%s\n", pwl);
+
+    /* Match empire. */
+    if ((!strncasecmp (name, pwl, STRNCMPT strlen (name))) && (pwl[j] == ':'))
+    {
+      if (newentry)
+        fprintf (pwf2, "%s\n", newentry);
+      matched = 1;
+    }
+    else
+    {
+      /* No match, so save line */
+      if (pwl[0])
+        fprintf (pwf2, "%s\n", pwl);
+    }
+  }
+  assert (0);
+  return (1);
+}
+
 int
 verify_password (player * P, const char *pw)
 {
   FILE *pwf;
-  char pwfpw[MAXTEXT], pwl[200];
+  char pwfpw[MAXTEXT], pwl[MAXSTRLEN];
   int j, k;
 
-  pwf = fopen ("races:passwords", "r");
+  memset (pwl, 0, MAXSTRLEN);
+  pwf = fopen (PASSWORDFILE, "r");
   if (pwf == NULL)
-    exit (MISSINGPW);			/* Urk. No file? */
-
+  {
+    error ("Password file missing!\n");
+    exit (MISSINGPW);           /* Urk. No file? */
+  }
+  j = strlen (P->name) + 1;     /* Needed later */
   for (;;)
   {
-    if (feof (pwf))		/* Player name not matched in file */
+    if (feof (pwf))             /* Player name not matched in file */
     {
+      fclose (pwf);
       if (strlen (pw) == 0)
-	return (0);		/* No password given, or req'd */
+        return (0);             /* No password given, or req'd */
       error ("Player name missing from password file\n");
       return (1);
-    }				/* Do the EOF check first, to deal with empty file and match on last */
+    }
+    /* Do the EOF check first, to deal with empty file and match on last */
     fscanf (pwf, "%s\n", pwl);
-    if (!strncasecmp (P->name, pwl, strlen (P->name)))	/* Match ... */
+
+    /* Match empire name check. */
+    if ((!strncasecmp (P->name, pwl, STRNCMPT strlen (P->name))) && (pwl[j - 1] == ':'))
     {
-      j = strlen (P->name) + 1;
       for (k = 0; pwl[k + j]; k++)
-	pwfpw[k] = pwl[k + j];
+        pwfpw[k] = pwl[k + j];
       pwfpw[k] = '\0';
-      if (strlen (pw) == strlen (pwfpw))	/* Potential match */
+      if (strlen (pw) == strlen (pwfpw))        /* Potential match */
       {
-	if (hstricmp (pw, pwfpw) != 0)
-	{
-	  sprintf (err_tmp, "Password mismatch\n");
-	  return (1);
-	}
-	return (0);		/* Matched */
+        fclose (pwf);
+        if (hstricmp (pw, pwfpw) != 0)
+        {
+          sprintf (err_tmp, "Password %s not matched for %s\n", pw, P->name);
+          return (1);
+        }
+        return (0);             /* Matched */
       }
       else
       {
-	sprintf (err_tmp, "Password mismatch\n");
-	return (1);
+        fclose (pwf);
+        sprintf (err_tmp, "Password %s not matched for %s\n", pw, P->name);
+        return (1);
       }
     }
   }
-  assert (0);			/* Should have been given a valid player name */
+  assert (0);                   /* Should have been given a valid player name */
+  return (1);
+}
+#endif
+
+#ifdef ZCMDOK
+int
+reset_email (char *name, const char newentry[])
+{
+  FILE *pwf, *pwf2;
+  char pwl[MAXTEXT + 2 + MAXSTRLEN];    /* BUG: multiple long email @'s */
+  int j;
+  int matched = 0;
+
+  memset (pwl, 0, 5 * MAXTEXT + 1);
+  pwf = fopen ("races:players", "r");
+  pwf2 = fopen ("players:races", "w");
+  if (pwf == NULL)
+  {
+    error ("Player file missing!\n");
+    exit (MISSINGRP);           /* Urk. No file? */
+  }
+  if (pwf2 == NULL)
+  {
+    error ("Unable to create temp password file!\n");
+    exit (MISSINGRP);           /* This is wrong, but probably will never happen? */
+  }
+  if (name[0])
+    j = strlen (name);
+  else
+    j = 0;                      /* Needed later */
+  for (;;)
+  {
+    /* Do the EOF check first, to deal with empty file and match on last */
+    fgets (pwl, MAXSTRLEN - 1, pwf);
+    pwl[strlen (pwl) - 1] = '\0';
+
+    if (feof (pwf))             /* Player name not matched in file */
+    {
+      if ((!matched) && newentry)
+      {
+        fprintf (pwf2, "%s\n", newentry);
+      }
+      fclose (pwf);
+      fclose (pwf2);
+      if (rename ("players:races", "races:players"))
+      {
+        sprintf (err_tmp, "Rename failed, error number %d\n", errno);
+        error (err_tmp);
+        exit (MISSINGRP);       /* Code here is wrong too, but close */
+      }
+      return (0);
+    }
+
+    /* Match empire. */
+    if ((!strncasecmp (name, pwl, STRNCMPT strlen (name))) && (pwl[j] == ':'))
+    {
+      if (newentry)
+        fprintf (pwf2, "%s\n", newentry);
+      matched = 1;
+    }
+    else
+    {
+      fprintf (pwf2, "%s\n", pwl);
+    }
+  }
+  assert (0);
+  return (1);
 }
 #endif
 
@@ -199,91 +350,143 @@ n2inputplayer (char *prompt)
   int i;
   player *p;
   char *nbuf;
-  char name[MAXTEXT + 1];
+  char name[MAXSTRLEN];
+  int quoted;                   /* For enabling "empire name" */
 
 #ifdef PASSWORDS
-  char password[MAXTEXT + 1];
+  char password[MAXSTRLEN];
 
 #endif
 
   printf ("%s%c", prompt, lf);
   for (;;)
-  {				/* Keep reading lines of input until EOF */
+  {                             /* Keep reading lines of input until EOF */
     gets (buf);
     if (feof (stdin))
       return (NULL);
 
     if (buf[0])
-    {		/* Non-empty line, so see if it has magic #galaxy cookie */
+    {
+      /* Non-empty line, so see if it has magic #galaxy cookie */
       if (!strncasecmp (buf, "#galaxy", 7))
       {
-	nbuf = strchr (buf, ' ');	/* Find first space */
-	if (nbuf == NULL)
-	{
-	  sprintf (err_tmp, "Null game- \"%s\"\n", buf);
-	  return (NULL);
-	}
-	for (; nbuf;)		/* Kill whitespace */
-	  if ((nbuf[0] == ' ') || (nbuf[0] == '\t'))
-	    nbuf++;
-	  else
-	    break;
+        nbuf = strchr (buf, ' ');       /* Find first space */
+        if (nbuf == NULL)
+        {
+          sprintf (err_tmp, "Null gamename- \"%s\"\n", buf);
+          return (NULL);
+        }
+        for (; nbuf;)           /* Kill whitespace */
+          if ((nbuf[0] == ' ') || (nbuf[0] == '\t'))
+            nbuf++;
+          else
+            break;
 
-	nbuf = strchr (nbuf, ' ');	/* Find 2nd wordbreak */
-	if (!nbuf)
-	  break;		/* TEMP BUGFIX */
-	nbuf++;
-	for (; nbuf;)
-	  if ((nbuf[0] == ' ') || (nbuf[0] == '\t'))
-	    nbuf++;
-	  else
-	    break;
+#ifdef GAMENAME
+        {
+          char gname[MAXSTRLEN];
+          char *j;
 
-	/* Now we grab the name.... */
-	for (i = 0; i < strlen (nbuf); i++)
-	{
-	  /* BUG: #galaxy game "empire name" not handled */
-	  if ((nbuf[i] != ' ') && (nbuf[0] != '\t'))
-	    name[i] = nbuf[i];
-	  else
-	    break;
-	}
-	name[i] = '\0';
-	nbuf += i;
-	for (; nbuf;)		/* Find 3rd break's whitespace end */
-	  if ((nbuf[0] == ' ') || (nbuf[0] == '\t'))
-	    nbuf++;
-	  else
-	    break;
-
-#ifdef PASSWORDS
-	/* Copy password entry */
-	for (i = 0; i < strlen (nbuf); i++)
-	  if ((nbuf[i] != ' ') && (nbuf[0] != '\t'))
-	    password[i] = nbuf[i];
-	  else
-	    break;
-	password[i] = '\0';
-
-	printf ("%s's password is %s\n", name, password);
+          /* copy the gamename, and then truncate the string at first
+             space or tab, then do comparison of gamename. */
+          hstrncpy (gname, nbuf, MAXSTRLEN);
+          for (j = gname; j; j++)
+          {
+            if ((j[0] == ' ') || (j[0] == '\t'))
+            {
+              j[0] = '\0';
+              break;
+            }
+          }
+          if (hstricmp (gname, gamename) != 0)
+          {
+            sprintf (err_tmp, "Gamename mismatch- \"%s\"\n", gname);
+            return (NULL);
+          }
+        }
 #endif
 
-	if ((p = nametop (players, name)) != 0)
-	{
+        /* have found & checked "#galaxy gamename" */
+        nbuf = strchr (nbuf, ' ');      /* Find 2nd wordbreak */
+        if (!nbuf)
+          break;                /* TEMP BUGFIX */
+        nbuf++;
+        for (; nbuf;)
+          if ((nbuf[0] == ' ') || (nbuf[0] == '\t'))
+            nbuf++;
+          else
+            break;
+
+        /* Now we look for and grab the playername.... */
+        if (nbuf[0] == '"')     /* "blah blah" */
+        {
+          nbuf++;
+          quoted = 1;
+        }
+        else
+          quoted = 0;
+
+        for (i = 0; i < strlen (nbuf); i++)
+        {
+          if ((nbuf[i] != ' ') && (nbuf[i] != '\t') && (nbuf[i] != '"'))
+            name[i] = nbuf[i];  /* Normal */
+          else
+          {
+            if ((nbuf[i] == ' ') && quoted)     /* quoted name, and space */
+              name[i] = '_';
+            else
+              break;
+          }
+        }
+        name[i] = '\0';
+        /* Clean up with quoted name */
+        if (quoted)
+        {
+          nbuf++;               /* advance by one, to dispose of trailing " */
+        }
 
 #ifdef PASSWORDS
-	  if (verify_password (p, password) != 0)
-	    return (NULL);
+        nbuf += i;
+        for (; nbuf;)           /* Find 3rd break's whitespace end */
+          if ((nbuf[0] == ' ') || (nbuf[0] == '\t'))
+            nbuf++;
+          else
+            break;
+
+        /* Copy password entry */
+        for (i = 0; i < strlen (nbuf); i++)
+          if ((nbuf[i] != ' ') && (nbuf[0] != '\t'))
+            password[i] = nbuf[i];
+          else
+            break;
+        password[i] = '\0';
 #endif
 
-	  return p;
-	}
-	sprintf (err_tmp, "Player \"%s\" not found.\n", buf);
-	return (NULL);
+        if ((p = nametop (players, name)) != 0)
+        {
+
+#ifdef PASSWORDS
+          if (verify_password (p, password) != 0)
+          {
+
+#ifdef PREPROC
+            error (err_tmp);
+            exit (BADPW);
+#else
+            fprintf (stderr, "%s", err_tmp);
+            return (NULL);
+#endif
+          }
+#endif
+
+          return p;
+        }
+        sprintf (err_tmp, "Player \"%s\" not found.\n", buf);
+        return (NULL);
       }
     }
   }
-  return (NULL);		/* BUG PROBLEM */
+  return (NULL);                /* BUG PROBLEM */
 }
 
 double
@@ -292,8 +495,8 @@ shipcost (shiptype * type, planet * p)
   double mass;
 
   mass = typemass (type);
-  assert (p->mat >= 0.0);	/* Stupidity check */
-  if (p->mat <= 0.005)		/* Round off error check */
+  assert (p->mat >= 0.0);       /* Stupidity check */
+  if (p->mat <= 0.005)          /* Round off error check */
     return ((INDPERSHIP + 1.0 / p->resources) * mass);
   if (p->mat > typemass (type))
     return (INDPERSHIP * mass);
@@ -314,9 +517,6 @@ setproduction (planet * p, int t)
       p->mat += (p->inprogress / shipcost (p->producingshiptype, p)) * mass;
     }
   }
-  /* So why doesn't this work? 
-     else
-     p->producingshiptype=(shiptype *)0; */
   p->inprogress = 0.0;
   p->producing = t;
 }
@@ -336,10 +536,6 @@ inputproduction (void)
   };
   int i;
 
-#ifdef INTERACTIVE
-LOOP:
-#endif
-
   printf ("Production type?%c", lf);
   getstr ();
   for (i = 0; i < sizeof (productionname) / sizeof (char *); i++)
@@ -347,28 +543,8 @@ LOOP:
     if (!hstricmp (productionname[i], buf))
       return i;
   sprintf (err_tmp, "\"%s\" is not a production type.\n", buf);
-
-#ifdef  INTERACTIVE
-  goto LOOP;
-#else
   error (err_tmp);
   return 0;
-#endif
-}
-
-double
-inputcomponent (char *s)
-{
-  double x;
-
-  printf ("%s%c", s, lf);
-  x = getf ();
-  if (!x)
-    return 0;
-
-  if (x < 1.0)
-    return (-1.0);
-  return x;
 }
 
 group *
@@ -406,11 +582,6 @@ LOOP:
   if (!g)
   {
     sprintf (err_tmp, "There is no group %d.\n", groupno);
-    /*error (err_tmp); */
-
-#ifdef  INTERACTIVE
-    goto LOOP;
-#endif
   }
   return g;
 }
@@ -427,61 +598,27 @@ inputcargotype (void)
     "emp",
   };
 
-#ifdef INTERACTIVE
-LOOP:
-#endif
-
   printf ("Cargo?%c", lf);
   getstr ();
   if (buf[0] == 0)
-    return -1;
+    return NOCARGO;
   buf[3] = 0;
   for (i = 0; i < sizeof (cargonames) / sizeof (char *); i++)
 
     if (!hstricmp (cargonames[i], buf))
       return i;
   sprintf (err_tmp, "\"%s\" is not a cargo type.\n", buf);
-
-#ifdef  INTERACTIVE
-  goto LOOP;
-#else
   return 0;
-#endif
 }
 
 void
-send (group * g, planet * p)
+Send (group * g, planet * p)
 {
   g->from = g->where;
   g->where = p;
   g->dist = dist (p, g->from);
 }
 
-shiptype *
-inputshiptype (player * P)
-{
-  shiptype *T;
-
-#ifdef  INTERACTIVE
-  for (;;)
-#endif
-
-  {
-    printf ("Ship Type?%c", lf);
-    getstr ();
-    if (!buf[0])
-      return 0;
-    for (T = shiptypes; T; T = T->next)
-      if (T->owner == P && !hstricmp (T->name, buf))
-	return T;
-    sprintf (err_tmp, "\"%s\" is not a ship type.\n", buf);
-
-#ifndef INTERACTIVE
-    return 0;
-#endif
-  }
-}
-
 void
 addcol (planet * p, double x)
 {
@@ -505,8 +642,12 @@ unloadgroup (group * g, player * P)
   x = g->load * g->ships;
   p = g->where;
 
-  assert (p->size);		/* can't unload onto size 0 systems; die
-				 * nothing if try */
+#ifndef FORECAST
+  /* can't unload onto size 0 systems; die nothing if try */
+  /* Sigh. R COL ZeroWorld  Is legal, and will crash the forecaster! */
+  /* This is rare, but can happen! */
+  assert (p->size);
+#endif
 
   switch (g->loadtype)
   {
@@ -524,14 +665,14 @@ unloadgroup (group * g, player * P)
     {
       if (p->owner == NULL)
       {
-	p->owner = P;
-	p->producing = PR_CAP;
+        p->owner = P;
+        p->producing = PR_CAP;
       }
       if (p->owner == P)
-	addcol (p, x);
+        addcol (p, x);
 
       else
-	p->mat += x * MATPERCOL;
+        p->mat += x * MATPERCOL;
     }
     break;
   default:
@@ -545,21 +686,24 @@ unloadgroup (group * g, player * P)
 void
 set_defend (player * P, int status, planet * p)
 {
-  planetdefense *df;
+  planetdefense *df, *df2;
 
   if (P)
   {
-    for (df = defender; df; df = df->next)
+    for (df = defender; df; df = df2)
+    {
+      df2 = df->next;
       if ((df->planet == p) && (df->defender == P))
-      {				/* They've changed a current one */
-	if (status == FLUSH)
-	  removelist (&defender, df);
-	else
-	  df->dipstat = status;
-	break;
+      {                         /* They've changed a current one */
+        if (status == FLUSH)
+          removelist (&defender, df);
+        else
+          df->dipstat = status;
+        break;
       }
+    }
     if (!df)
-    {				/* Or added one */
+    {                           /* Or added one */
       df = alloc (sizeof (planetdefense));
       df->dipstat = status;
       df->planet = p;
@@ -567,14 +711,17 @@ set_defend (player * P, int status, planet * p)
       addlist (&defender, df);
     }
   }
-  else				/* Or cleaning one up */
-    for (df = defender; df; df = df->next)
+  else                          /* Or cleaning one up */
+    for (df = defender; df; df = df2)
+    {
+      df2 = df->next;
       if (df->planet == p)
-	removelist (&defender, df);
+        removelist (&defender, df);
+    }
 }
 
 void
-enterturn (void)
+enterturn (int turnrun)
 {
   player *P;
 
@@ -582,6 +729,7 @@ enterturn (void)
 #ifdef PREPROC
   P = n2inputplayer ("");
   n_order = 0;
+  bad = 0;
 #else
   P = n2inputplayer ("Player?");
 #endif
@@ -605,29 +753,36 @@ enterturn (void)
 #endif
 
 #ifndef PREPROC
-  if (P->lastorders > turn) 
+  if (abs (P->lastorders) > turn)
   {
-    sprintf (err_tmp,"Multiple orders detected for %s!\n",P->name);
+    sprintf (err_tmp, "Multiple orders detected for %s!\n", P->name);
     error (err_tmp);
+    exit (DUPORDERS);
   }
 #endif
 
-  P->lastorders = turn + 1;
+  if (P->lastorders < 0)
+    P->lastorders = -(turn + 1);
+  else
+    P->lastorders = turn + 1;
 
   for (;;)
   {
 
     orders[0] = '\0';
 
-#ifndef PREPROC
-    printf ("%s>%c", P->name, lf);
-    fflush (stdout);
-#else
+  LOOP:
+
+#ifdef PREPROC
+    /* Do this here so we count blank lines and such. */
     n_order++;
     printf ("%s:%d>\n", P->name, n_order);
+#else
+    printf ("%s>%c", P->name, lf);
+    fflush (stdout);
+    /* Ignoring blank lines here is OK. */
 #endif
 
-  LOOP:
     gets (buf);
     if (feof (stdin))
     {
@@ -636,9 +791,20 @@ enterturn (void)
     }
     append_cmd (orders, buf);
     if (!buf[0])
-      goto LOOP;		/* Blank line */
+      goto LOOP;                /* Blank line */
     if ((strlen (buf) == 1) && (buf[0] == ' ' || buf[0] == '\t'))
-      goto LOOP;		/* Space */
+      goto LOOP;                /* Space/TAB */
+
+#ifdef PREPROC
+    assert (bad < TOOMANYBAD + 1);      /* Sanity check */
+    if (bad == TOOMANYBAD)
+    {
+      bad--;
+      error ("Too many bad commands, giving up on remaining orders\n");
+      strcpy (buf, "#end");
+    }
+#endif
+
     switch (tolower (buf[0]))
     {
     case 'a':
@@ -659,35 +825,18 @@ enterturn (void)
     case 'f':
       f_cmd (buf, P);
       break;
-
-#ifdef V3
     case 'g':
       g_cmd (buf, P);
-#else
+      break;
     case 'h':
       h_cmd (buf, P);
-#endif
-
-      break;
-
-#ifdef V3
-    case 'h':			/* def mod */
-      h_cmd (buf, P);
-#else
-    case 'g':			/* def mod */
-      g_cmd (buf, P);
-#endif
-
       break;
     case 'i':
       i_cmd (buf, P);
       break;
-
-#ifdef UPGRADE
     case 'j':
       j_cmd (buf, P);
       break;
-#endif
 
 #ifdef KCMD
     case 'k':
@@ -710,21 +859,6 @@ enterturn (void)
     case 'p':
       p_cmd (buf, P);
       break;
-
-#ifndef UPGRADE
-    case 'q':
-
-#ifdef FORECAST
-      if (P->opts2 & FCAST)
-      {
-	while (P->lastorders > turn)
-	  go ();
-	forecast_report (P);
-      }
-#endif
-
-      return;
-#endif
     case 'r':
       r_cmd (buf, P);
       break;
@@ -734,7 +868,6 @@ enterturn (void)
     case 't':
       t_cmd (buf, P);
       break;
-
     case 'u':
       u_cmd (buf, P);
       break;
@@ -744,6 +877,19 @@ enterturn (void)
     case 'x':
       x_cmd (buf, P);
       break;
+
+#ifdef PASSWORDS
+    case 'y':
+      y_cmd (buf, P);
+      break;
+#endif
+
+#ifdef ZCMDOK
+    case 'z':
+      z_cmd (buf, P);
+      break;
+#endif
+
     case ';':
       break;
     case '#':
@@ -751,24 +897,24 @@ enterturn (void)
       {
 
 #ifdef FORECAST
-	if (P->opts2 & FCAST)
-	{
-	  while (P->lastorders > turn)
-	    go ();
-	  forecast_report (P);
-	}
+        if ((P->opts2 & FCAST) && (turnrun == RUN))
+        {
+          while (abs (P->lastorders) > turn)
+            go ();
+          forecast_report (P);
+        }
 #endif
 
-	return;
+        return;
       }
       break;
 
     case ' ':
     case '\t':
 
-#ifdef PREPROC			/* Safe to ignore, but flag just in case */
+#ifdef PREPROC                  /* Flag, in case they do <space>order */
       sprintf (err_tmp,
-	    "Warning! Space caused this command to be ignored '%s'\n", buf);
+            "Warning! Space caused this command to be ignored '%s'\n", buf);
       error (err_tmp);
 #endif
 
@@ -789,20 +935,10 @@ enterturn (void)
       printf ("D = Design Ship Type\n");
       printf ("E = Erase Ship Type\n");
       printf ("F = Full system request\n");
-
-#ifndef V3
-      printf ("G = Declare Defensive\n");
-      printf ("H = Upgrade\n");
-#else
       printf ("G = Upgrade\n");
       printf ("H = Declare Defensive\n");
-#endif
-
       printf ("I = Intercept\n");
-
-#ifdef UPGRADE
       printf ("J = Join\n");
-#endif
 
 #ifdef KCMD
       printf ("K = Planet protection\n");
@@ -819,6 +955,15 @@ enterturn (void)
       printf ("U = Unload Cargo\n");
       printf ("W = Declare War\n");
       printf ("X = Scrap\n");
+
+#ifdef PASSWORDS
+      printf ("Y = Reset Password\n");
+#endif
+
+#ifdef ZCMDOK
+      printf ("Z = Change Email\n");
+#endif
+
 #endif
     }
   }
